@@ -49,7 +49,7 @@ void fluid_shell_settings(fluid_settings_t* settings)
 
 fluid_cmd_t fluid_commands[] = {
   { "help", "general", (fluid_cmd_func_t) fluid_handle_help, NULL,
-    "help                       Print command summary. 'help help': Show more help topics" },
+    "help                       Command summary. 'help help' for more help topics" },
   { "quit", "general", (fluid_cmd_func_t) fluid_handle_quit, NULL,
     "quit                       Quit the synthesizer" },
   { "noteon", "event", (fluid_cmd_func_t) fluid_handle_noteon, NULL,
@@ -63,9 +63,9 @@ fluid_cmd_t fluid_commands[] = {
   { "select", "event", (fluid_cmd_func_t) fluid_handle_select, NULL,
     "select chan sfont bank prog  Combination of bank-select and program-change" },
   { "load", "general", (fluid_cmd_func_t) fluid_handle_load, NULL,
-    "load file [reset] [bank_offset]   Load a SoundFont (reset=0|1, default 1; bank_offset=n, default 0)" },
+    "load file [reset] [bankofs] Load SoundFont (reset=0|1, def 1; bankofs=n, def 0)" },
   { "unload", "general", (fluid_cmd_func_t) fluid_handle_unload, NULL,
-    "unload id [reset]          Unload the SoundFont with the specified ID (reset=0|1, default 1)"},
+    "unload id [reset]          Unload SoundFont by ID (reset=0|1, default 1)"},
   { "reload", "general", (fluid_cmd_func_t) fluid_handle_reload, NULL,
     "reload id                  Reload the SoundFont with the specified ID" },
   { "fonts", "general", (fluid_cmd_func_t) fluid_handle_fonts, NULL,
@@ -116,7 +116,7 @@ fluid_cmd_t fluid_commands[] = {
   { "dumptuning", "tuning", (fluid_cmd_func_t) fluid_handle_dumptuning, NULL,
     "dumptuning bank prog       Print the pitch details of the tuning" },
   { "reset", "general", (fluid_cmd_func_t) fluid_handle_reset, NULL,
-    "reset                      Send a system reset (all notes off, reset controllers)" },
+    "reset                      System reset (all notes off, reset controllers)" },
   { "set", "settings", (fluid_cmd_func_t) fluid_handle_set, NULL,
     "set name value             Set the value of a controller or settings" },
   { "get", "settings", (fluid_cmd_func_t) fluid_handle_get, NULL,
