@@ -319,12 +319,23 @@ int fluid_ramsfont_add_izone(fluid_ramsfont_t* sfont,
 }
 
 int fluid_ramsfont_remove_izone(fluid_ramsfont_t* sfont,
-        unsigned int bank, unsigned int num, fluid_sample_t* sample) {
+         unsigned int bank, unsigned int num, fluid_sample_t* sample) {
+	int err;
 	fluid_rampreset_t* preset = fluid_ramsfont_get_preset(sfont, bank, num);
 	if (preset == NULL) {
-			return FLUID_FAILED;
+		return FLUID_FAILED;
 	}
-	return fluid_rampreset_remove_izone(preset, sample);
+
+	// Fixed a crash bug : remove the sample from the sfont list after
+	// removing the izone (aschmitt august 2005)
+	err = fluid_rampreset_remove_izone(preset, sample);
+	if (err != FLUID_OK)
+		return err;
+
+	// now we must remove the sample from sfont->sample
+	sfont->sample = fluid_list_remove(sfont->sample, sample);
+
+	return FLUID_OK;
 }
 
 
